#ifndef WHLIST_DBUS_INTERFACE_H
#define WHLIST_DBUS_INTERFACE_H

#include <sys/queue.h>

//kysec执行控制标记类型字符串
#define KYSEC_EXECTL_LABEL_ORIGINAL (char*)"original"
#define KYSEC_EXECTL_LABEL_VERIFIED (char*)"verified"
#define KYSEC_EXECTL_LABEL_KYSOFT (char*)"kysoft"
#define KYSEC_EXECTL_LABEL_TRUSTED (char*)"trusted"
#define KYSEC_EXECTL_LABEL_PARENT (char*)"parent"
#define KYSEC_EXECTL_LABEL_SCRIPT (char*)"script"
#define KYSEC_EXECTL_LABEL_UNKNOWN (char*)"unknown"

//kysec文件保护标记类型字符串
#define KYSEC_FILEPRO_LABEL_READONLY (char*)"readonly"
#define KYSEC_FILEPRO_LABEL_NONE (char*)"none"

//kysec功能类型
enum e_kysec_type {
	kysec_exectrl = 0,		//执行控制
	kysec_filepro,			//文件保护
	kysec_kmodpro			//内核模块保护
};

enum e_kysec_whlist_operate	{
	kysec_operate_add = 0,
	kysec_operate_delete,
	kysec_operate_update
};

//kysec执行控制标记类型
enum e_exectl_sign_type {
	exectl_sign_verified = 0,	//认证标记
	exectl_sign_original,		//初始标记
	exectl_sign_kysoft,		//麒麟软件标记
	exectl_sign_trusted,		//信任标记	
	exectl_sign_unknown,		//未知、篡改标记
	exectl_sign_parent,		//dpkg类标记
	exectl_sign_script		//脚本解释器类标记
};

static const char *exectl_sign_str[] = {
    "verified",
    "original",
    "kysoft",
    "trusted",
    "unknown",
    "parent",
    "script"
};

//kysec文件保护标记类型
enum e_filepro_sign_type {
	filepro_sign_readonly = 0,	//只读保护
	filepro_sign_none		//无保护或被篡改
};

//白名单数据库记录
struct db_object {
	char *fpath;			//文件路径
	char *hash;			//摘要值
	int type;			//文件类型
	int exectl;			//执行控制标记类型
	int filepro;			//文件保护标记类型
	int rmmodpro;			//是否内核防卸载
	int status;			//数据项状态
};

/**
 * @brief 单链表结构体定义
 *
 * 单链表结构体名字是object_list，链表成员类型是object_entry
 */
SLIST_HEAD(object_list, object_entry);
struct object_entry {
	struct db_object object;
	SLIST_ENTRY(object_entry) entries;
};

//内核模块防卸载白名单数据库记录
struct kmod_object {
	char *kmod_name;			//内核模块名称
	int status;			//防卸载状态
};

/**
 * @brief 单链表结构体定义
 *
 * 单链表结构体名字是object_list，链表成员类型是object_entry
 */
SLIST_HEAD(kmod_list, kmod_entry);
struct kmod_entry {
	struct kmod_object object;
	SLIST_ENTRY(kmod_entry) entries;
};

#ifdef __cplusplus
extern "C" {
#endif

/*
 * check whether the current user is secadm.
 * return values:
 * 0 : not secadm
 * 1 : secadm
 */
int kysec_whlist_is_secadm(void);

/*
 * check whether the current user is auditadm.
 * return values:
 * 0 : not auditadm
 * 1 : auditadm
 */
int kysec_whlist_is_auditadm(void);

/*
 * 获取kysec状态
 * 参数：
 * 返回值：
 *      0： KYSEC已禁用
 *      2： KYSEC强制模式
 *      4：KYSEC软模式
 *      其他：获取失败
 */
int kysec_whlist_getstatus();

/*
 * 设置kysec状态
 * 参数：	status     2： KYSEC强制模式      4：KYSEC软模式
* 返回值：
 *      0： 状态设置成功
 *      其他：设置失败
 */
int kysec_whlist_setstatus(int status);

/*
 * 获取kysec子模块状态
 * 参数：	func     0：执行控制      1：网络管控		2：文件保护		3：内核模块保护		4：三权分立
 * 返回值：
 *      0： 已关闭
 *      1： 常规模式
 *      2：	警告模式
 *      其他：获取失败
 */
int kysec_whlist_get_func_status(int func);

/*
 * 设置kysec子模块状态
 * 参数：	func     0：执行控制      1：网络管控		2：文件保护		3：内核模块保护		4：三权分立
 * 			status     0：关闭      1：正常模式			2：警告模式
 *  * 返回值：
 *      0： 设置成功
 *      其他：设置失败
 */
int kysec_whlist_set_func_status(int func, int status);

int kysec_whlist_parse_exectl_type(char *exectl);

/*
 * dpkg装包执行控制白名单添加接口
 * 参数：
 *      path：需要添加到白名单列表的文件路径，该文件为可执行文件、shell脚本、python脚本、
 *            so动态链接库、ko内核模块时设置标记并添加白名单，否则，仅设置标记
 * 返回值：
 *      0： 添加成功或未开启kysec
 *      1： 认证失败，没有权限添加
 *      -1：其他添加失败
 *      -2：参数错误，路径不存在或参数不对
 */
int kysec_whlist_exectl_add_for_dpkg(char* path);

/*
 * dpkg装包执行控制白名单添加接口
 * 参数：
 *      path：需要添加到白名单列表的文件路径，该文件为可执行文件、shell脚本、python脚本、
 *            so动态链接库、ko内核模块时设置标记并添加白名单，否则，仅设置标记
 * 返回值：
 *     	>= 0： 添加成功数量
 *      -1： 认证失败，没有权限添加 
 *      -2：参数错误，路径不存在或参数不对
 * 		-3：其他添加失败
 */
int kysec_whlist_exectl_multi_add_for_dpkg(char* path_list[], int count);

/*
 * dpkg装包重打readonly接口
 * 参数： 
 * 返回值：
 *      0： 设置成功
 *      其他： 失败
 */
int kysec_whlist_protect_set_for_dpkg();

/*
 * dpkg装包重打script接口
 * 参数：
 * 返回值：
 *      0： 设置成功
 *      其他： 失败
 */
int kysec_whlist_script_set_for_dpkg();

/*
 * 执行控制白名单添加接口
 * 参数：
 *      path：需要添加到白名单列表的文件路径，该文件必须为可执行文件、shell脚本、python脚本、
 *            so动态链接库、ko内核模块；
 * 返回值：
 *      0： 添加成功
 *      1： 认证失败，没有权限添加
 *      -1：其他添加失败
 *      -2：参数错误，路径不存在或参数不对
 *      -3：文件类型不对，非可执行文件、shell脚本、python脚本、so动态链接库、ko内核模块
 */
int kysec_whlist_exectl_add(char* path);

/*
 * 执行控制白名单更新接口
 * 参数：
 *      path：需要更新到白名单列表的文件路径，该文件必须为可执行文件、shell脚本、python脚本、
 *            so动态链接库、ko内核模块；如果是该文件存在则更新，如果不存在则添加；
 * 返回值：
 *      0： 更新成功
 *      1： 认证失败，没有权限
 *      -1：其他失败
 *      -2：参数错误，路径不存在或参数不对
 *      -3：文件类型不对，非可执行文件、shell脚本、python脚本、so动态链接库、ko内核模块
 */
int kysec_whlist_exectl_update(char* path);

/*
 * 执行控制白名单移除接口
 * 参数：
 *      path：需要从白名单列表删除的文件路径，该文件必须为可执行文件、shell脚本、python脚本、
 *            so动态链接库、ko内核模块；
 * 返回值：
 *      0： 删除成功
 *      1： 认证失败，没有权限
 *      -1：其他失败
 *      -2：参数错误，路径不存在或参数不对
 *      -3：文件类型不对，非可执行文件、shell脚本、python脚本、so动态链接库、ko内核模块
 */
int kysec_whlist_exectl_remove(char* path);

/*********************************************************************************/

/**
 * 根据功能类型添加白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * 返回值：
 *      0： 成功
 *      其他： 失败
 */
int kysec_whlist_add(int kysec_type, char *path);

/*
 * 涉密场景：set kysec protect label to file.
 * return values:
 * 0 : success
 * other : failed
*/
int kysec_whlist_add_sm(int kysec_type, char *path);

/**
 * 根据功能类型批量添加白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * 返回值：
 *      >=0： 成功添加数量
 *      其他： 失败
 */
int kysec_whlist_multi_add(int kysec_type, char *paths[], int count);

/**
 * 根据功能类型删除白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * 返回值：
 *      0： 成功
 *      其他： 失败
 */
int kysec_whlist_delete(int kysec_type, char *path);

/*
 * 涉密场景：set kysec protect label to file.
 * return values:
 * 0 : success
 * other : failed
*/
int kysec_whlist_delete_sm(int kysec_type, char *path);

/**
 * 根据功能类型批量删除白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * 返回值：
 *      >=0： 成功删除数量
 *      其他： 失败
 */
int kysec_whlist_multi_delete(int kysec_type, char *paths[], int count);

/**
 * 根据功能类型更新白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * 返回值：
 *      0： 成功
 *      其他： 失败
 */
int kysec_whlist_update(int kysec_type, char *path);

/**
 * 根据功能类型批量更新白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * 返回值：
 *      >=0： 成功更新数量
 *      其他： 失败
 */
int kysec_whlist_multi_update(int kysec_type, char *paths[], int count);

/**
 * 更新内核模块防卸载状态
 * 返回值：
 *      0： 成功
 *      其他： 失败
 */
int kysec_whlist_update_rmmodpro(int status, char *path);

/**
 * 涉密场景：更新内核模块防卸载状态
 * 返回值：
 *      0： 成功
 *      其他： 失败
 */
int kysec_whlist_update_rmmodpro_sm(int status, char *path);

/**
 * 将程序包内所有文件添加白名单
 * 返回值：
 *      >=0： 成功添加文件数量
 *      其他： 失败
 */
int kysec_whlist_add_by_package(const char *package);

/**
 * 将程序包内所有文件更新白名单
 * 返回值：
 *      >=0： 成功更新文件数量
 *      其他： 失败
 */
int kysec_whlist_update_by_package(const char *package);

/**
 * 将程序包内所有文件删除白名单
 * 返回值：
 *      >=0： 成功删除文件数量
 *      其他： 失败
 */
int kysec_whlist_delete_by_package(const char *package);

/**
 * 将目录内符合符合文件类型所有文件添加对应白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * obj_type: program 可执行文件； library 动态库； script 可执行脚本； kmodule 内核模块
 * dpath： 目录绝对路径
 * 返回值：
 *      >=0： 成功添加文件数量
 *      其他： 失败
 */
int kysec_whlist_add_by_dir(int kysec_type, const char *obj_type, const char *dpath);

/*
 * 涉密场景：set kysec protect label to dir.
 * return values:
 * 0 : success
 * other : failed
*/
int kysec_whlist_add_by_dir_sm(int kysec_type, const char *obj_type, const char *dpath);

/**
 * 将目录内所有文件更新白名单
 * 返回值：
 *      >=0： 成功更新文件数量
 *      其他： 失败
 */
int kysec_whlist_update_by_dir(int kysec_type, const char *obj_type, const char *dpath);

/**
 * 将目录内所有文件删除白名单
 * 返回值：
 *      >=0： 成功删除文件数量
 *      其他： 失败
 */
int kysec_whlist_delete_by_dir(int kysec_type, const char *obj_type, const char *dpath);

/*
 * 涉密场景：set kysec protect label to dir.
 * return values:
 * 0 : success
 * other : failed
*/
int kysec_whlist_delete_by_dir_sm(int kysec_type, const char *obj_type, const char *dpath);

/**
 * 根据功能类型获取白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 */
int kysec_whlist_load(int kysec_type, const char *key, struct object_list *objects);

/**
 * 涉密场景：根据功能类型获取白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 */
int kysec_whlist_load_sm(int kysec_type, const char *key, struct object_list *objects);

/**
 * 获取所有白名单
 */
int kysec_whlist_load_all(struct object_list *objects);

/**
 * 根据功能类型导入白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 */
int kysec_whlist_import(int kysec_type,  char *file);

/**
 * 根据功能类型导出白名单
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 */
int kysec_whlist_export(int kysec_type,  char *file);

/**
 * 释放白名单列表
 */
void kysec_whlist_objects_free(struct object_list *objects);

/**
 * 获取白明单操作最后错误
 */
char *kysec_whlist_last_err();

/**
 * 根据功能类型、文件类型获取白名单数量
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * obj_type： program; library; script; kmodule other
 */
int kysec_whlist_get_count_by_obj_type(int kysec_type, const char *obj_type, const char *key);

/**
 * 根据功能类型、文件类型获取白名单列表
 * kysec_type： 0 执行控制； 1 文件保护； 2 内核模块保护
 * obj_type： program；library；script；kmodule；other
 */
int kysec_whlist_load_by_obj_type(int kysec_type, const char *obj_type, const char *key, struct object_list *objects);

/**
 * 根据文件绝对路径获取文件白名单信息
 */
int kysec_whlist_load_by_path(const char *path, struct db_object *object);

/**
 * 根据文件路径获取hash值
 */
char *kysec_whlist_get_hash_by_path(char *fpath);

/**
 * 获取内核模块防卸载白名单
 */
int kysec_kmod_whlist_load(const char *key, struct kmod_list *objects);

/**
 * 涉密场景：获取内核模块防卸载白名单
 */
int kysec_kmod_whlist_load_sm(const char *key, struct kmod_list *objects);

/**
 * 释放白名单列表
 */
void kysec_kmod_whlist_objects_free(struct kmod_list *objects);


/* udeblist.xml 存放的是软件防卸载黑白名单信息 */

/*
 * 创建软件包防卸载xml文档
 * 返回值:     -1:出错 0:文档已经存在 n(>0):新建xml文档，n个字节被写入文档
 */
int kysec_udeblist_create(void); 

/*
 * 添加一个黑白名单节点信息到xml文件中
 * @bw_value: 黑名单或白名单节点("white" or "black"，其他值不合法)
 * @debname:    添加到黑名单或者白名单的包名；
 * 返回值:      0:成功，-1:出错, 1:软件包已经在列表
 */     
int kysec_udeblist_add(char *bw_value, char* debname);

/*
 * 删除一个黑白名单节点信息
 * @bw_value:   黑名单或白名单节点("white" or "black", 其它值不合法)
 * @debname:    删除黑名单或者白名单的包名；
 * 返回值:      0:不存在，-1:出错，1成功;
 */
int kysec_udeblist_del(char *bw_value, char *debname);

/*
 * 清空列表
 *　返回值：-1失败，0 清空成功
 */
int kysec_udeblist_clear(void);

/*
 * 获取黑白名开关标识
 * 返回值：0:未开启黑白名单，1：开启白名单，2：开启黑名单, -1:出错
 */
int kysec_udeblist_get_root_distinguish(void);

/*
 * 设置黑白名单开关标识
 * @value: 0:不开启黑白名单，1：开启白名单，2：开启黑名单， 其它值非法
 * 返回值：0:成功，非0:失败
 */
int kysec_udeblist_set_root_distinguish(int value);

/*
 * 获取黑名单、白名单节点数据
 * @bw_value: 黑名单节点或者白名单节点
 * 返回值：包含黑名单或者白名单节点中包名，count为列表中包的个数,返回的二级指针>需要自己释放内存
 */
char **kysec_udeblist_read_all_node_with_uid(char *bw_value, int *count);


/* sm_udeblist.xml 存放的是软件防卸载黑白名单信息 */

/*
 * 创建sm_udeblist.xml文档
 * 返回值:     -1:出错 0:文档已经存在 n(>0):新建xml文档，n个字节被写入文档
 */
int kysec_udeblist_create_sm(void);

/*
 * 添加一个黑白名单节点信息到sm配置文件xml文件中
 * @bw_value:   黑名单或白名单节点("white" or "black", 其它值不合法)
 * @debname:    添加到黑名单或者白名单的包名；
 * 返回值:      0:成功，-1:出错, 1：软件包已经在列表
 */
int kysec_udeblist_add_sm(char *bw_value, char *debname);

/*
 * 删除一个涉密配置文件黑白名单节点信息
 * @bw_value:   黑名单或白名单节点("white" or "black", 其它值不合法)
 * @debname:    同时删除通用和sm配置文件中黑名单或者白名单的包名；
 * 返回值:      0:不存在，-1:出错，1成功;
 */
int kysec_udeblist_del_sm(char *bw_value, char *debname);

/*
 * 清空sm配置文件节点
 *　返回值：-1失败，0 清空成功
 */
int kysec_udeblist_clear_sm(void);

/*
 * 获取sm配置文件黑白名开关标识
 * 返回值：0:未开启黑白名单，1：开启白名单，2：开启黑名单, -1:出错
 */
int kysec_udeblist_get_root_distinguish_sm(void);

/*
 * 设置sm配置文件黑白名开关标识
 * @value: 0:不开启黑白名单，1：开启白名单，2：开启黑名单， 其它值非法
 * 返回值：0:成功，非0:失败
 */
int kysec_udeblist_set_root_distinguish_sm(int value);

/*
 * 获取sm配置文件黑名单、白名单节点数据
 * @bw_value: 黑名单节点或者白名单节点
 * 返回值：包含黑名单或者白名单节点中包名，count为列表中包的个数,返回的二级指针需要自己释放内存
 */
char **kysec_udeblist_read_all_node_with_uid_sm(char *bw_value, int *count);

/*
 * 设置全局用户桌面壁纸
 * @picture_path:    图片路径；
 * 返回值：-1失败，0 成功
 */         
int kysec_bgset_set(const char* picture_path);

/*
 * 取消全局用户桌面壁纸设定
 *　返回值：-1失败，0 成功
 */  
int kysec_bgset_unset();

/*
 * 执行控制白名单添加接口，分析可执行程序的动态库，若动态库为unknown标记，在加白的同时，给依赖的动态库加白
 * 参数：
 *      path：需要添加到白名单列表的文件路径
 * 返回值：
 *      0： 添加成功
 *      1： 认证失败，没有权限添加
 *      -1：其他添加失败
 *      -2：参数错误，路径不存在或参数不对
 */
int kysec_whlist_ldd_add(const char *path);

int kysec_exectl_ldd_set(char *path, char *kysec_label);

#ifdef __cplusplus
}
#endif

#endif // WHLIST_DBUS_INTERFACE_H
